/*
  Written by Darrell Mozingo, 2/4/04
  Modified 2/26/04: - handles a cleared eventlog properly now
					- added ability to negate the event's id and description fields
*/

using System;
using System.Diagnostics;

// Custom Namespaces
using CommandLine.Utility;

namespace NRPE.Eventlog
{
	class EventlogPlugin
	{
		private static void writeUsage(string strAppName)
		{
			Console.WriteLine("Usage: {0} -e eventlog  -t entryType [--help -h -m minutes\n\t\t\t\t-d description -i eventID]", strAppName);
			Console.WriteLine("\t-e = Must be either Application, Security, or System (or others if\n\t     you have 3rd party programs installed)");
			Console.WriteLine("\t-t = Must be either Error, Information, Warning, Success Audit,\n\t     Failure Audit, or All, depending on which type of\n\t     messages you want returned");
			Console.WriteLine("\t-m = Minutes to check back in the eventlog. Defaults to 5 minutes");
			Console.WriteLine("\t-d = String used to filter out certain messages from being returned\n\t     based on their description, in the form \"string1,string2,string3\"");
			Console.WriteLine("\t-i = EventID's to filter out in the form \"1111,2222,3333\"");
			Console.WriteLine("\t-nd = Negate the description filter, that is, instead of filtering\n\t     based on the strings you pass in, the plugin\n\t     ONLY returns the event's containing strings you\n\t     pass in");
			Console.WriteLine("\t-ni = Same as the -nd option, but for EventID's");
			Console.WriteLine("\t--help, -h = Prints this usage statement");
			Console.WriteLine();
		}

		static bool filterEvent(bool bNegDesc, bool bNegID, string strEventLogMessage, int intEventID, string [] descriptionFilters, string [] idFilters)
		{
			bool bFilteredOut = false;			

			if(descriptionFilters != null && idFilters != null)
			{
				bool bFilteredWithDescription = false;

				foreach(string description in descriptionFilters)
				{
					if(strEventLogMessage.IndexOf(description.ToUpper()) != -1)
					{
						bFilteredWithDescription = (bNegDesc ? false : true);
						bFilteredOut = (bNegDesc ? false : true);
						break;
					}
				}

				if(!bFilteredWithDescription)
				{
					foreach(string eventID in idFilters)
					{
						if(intEventID == Convert.ToInt32(eventID))
						{
							bFilteredOut = (bNegID ? false : true);
							break;
						}
					}
				}
			}
			else if(descriptionFilters != null && idFilters == null)
			{
				foreach(string description in descriptionFilters)
				{
					if(strEventLogMessage.IndexOf(description.ToUpper()) != -1)
					{
						bFilteredOut = (bNegDesc ? false : true);
						break;
					}
				}
			}
			else if(descriptionFilters == null && idFilters != null)
			{
				foreach(string eventID in idFilters)
				{
					if(intEventID == Convert.ToInt32(eventID))
					{
						bFilteredOut = (bNegID ? false : true);
						break;
					}
				}
			}

			return bFilteredOut;
		}

		[STAThread]
		static int Main(string[] args)
		{
			int intReturnValue = 3;
			Arguments CommandLine = new Arguments(args);

			string [] strDirectories = Environment.GetCommandLineArgs()[0].Split(new char [] { '\\' });
			string strAppName = strDirectories[strDirectories.Length - 1];
		
			if(CommandLine.ArgLength < 1 || CommandLine.ArgLength > 5 || CommandLine["e"] == null || 
				CommandLine["help"] != null || CommandLine["h"] != null || CommandLine["t"] == null)
			{
				writeUsage(strAppName);
				return intReturnValue;
			}

			string strEventLog = CommandLine["e"];

			if(!EventLog.Exists(strEventLog, "."))
			{
				Console.WriteLine("UNKNOWN: Eventlog \"{0}\" does not exist!", strEventLog);
				return intReturnValue;
			}

			EventLog eventLog = new EventLog();
			eventLog.Log = strEventLog;
			eventLog.MachineName = ".";

			EventLogEntryCollection eventLogEntryCollection = eventLog.Entries;

			string strEventType = CommandLine["t"];
			string [] strDescriptionFilters = null;
			string [] strEventIDFilters = null;
			int intMinutesBack = 5;
			bool bNegDescription = (CommandLine["nd"] == "true" ? true : false);
			bool bNegID = (CommandLine["ni"] == "true" ? true : false);
			string strReturn = "UNKNOWN: An error occured finding entries in the " + strEventLog + " log.";

			if(CommandLine["m"] != null && CommandLine["m"] != "true")
				intMinutesBack = Convert.ToInt32(CommandLine["m"]);

			if(CommandLine["d"] != null && CommandLine["d"] != "true")
				strDescriptionFilters = CommandLine["d"].Split(new char [] { ',' });

			if(CommandLine["i"] != null && CommandLine["i"] != "true")
				strEventIDFilters = CommandLine["i"].Split(new char [] { ',' });

			if(eventLogEntryCollection.Count == 0)
			{
				strReturn = "OK: No entries in the " + strEventLog + " log as it's been cleared recently.";
				intReturnValue = 0;
			}

			for(int i = eventLogEntryCollection.Count - 1; i >= 0; i--)
			{
				EventLogEntry eventLogEntry = eventLogEntryCollection[i];

				long lngElapsedTime = eventLogEntry.TimeWritten.AddMinutes(intMinutesBack).Ticks - DateTime.Now.Ticks;

				if(eventLogEntry.EntryType.ToString().ToUpper() == strEventType.ToUpper() && lngElapsedTime >= 0)
				{
					string eventLogMessage = eventLogEntry.Message.Trim();
					int eventLogID = eventLogEntry.EventID;

					if(!filterEvent(bNegDescription, bNegID, eventLogMessage.ToUpper(), eventLogID, strDescriptionFilters, strEventIDFilters))
					{
						strReturn = eventLogMessage + " ** EventID: " + eventLogID;
						intReturnValue = 2;
						break;
					}
				}
				else
				{
					strReturn = "OK: No entries in the " + strEventLog + " log recently.";
					intReturnValue = 0;
				}
			}
			
			Console.WriteLine(strReturn);
			eventLog.Close();

			return intReturnValue;
		}
	}
}